==> klayout-0.9.tar.gz: klayout-0.9/src/layProgress.cc <==

/*

  KLayout Layout Viewer
  Copyright (C) 2006 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include <QtCore/QMutex>
#include <QtCore/QThreadStorage>
#include <QtGui/QToolButton>
#include <QtGui/QLabel>
#include <QtGui/QPainter>
#include <QtGui/QHBoxLayout>
#include <QtGui/QApplication>

#include <stdlib.h>
#include <math.h>

#include <layProgress.h>
#include <layLayoutCanvas.h>
#include <tlProgress.h>

namespace lay
{

// --------------------------------------------------------------------

class ProgressBar
  : public QWidget 
{
public:
  ProgressBar (QWidget *parent, const char *name = "");

  void set_value (double v, const std::string &value);

  QSize sizeHint () const 
  {
    return QSize (m_width, 1);
  }

  QSize minimumSizeHint () const 
  {
    return QSize (m_width, 1);
  }

private:
  double m_value;
  std::string m_value_string;
  int m_width;
  int m_length;
  int m_fw;
  int m_bw;

  void paintEvent (QPaintEvent *event); 
  void resizeEvent (QResizeEvent *event); 
};

ProgressBar::ProgressBar (QWidget *parent, const char *name)
  : QWidget (parent, name),
    m_value (0.0), m_width (64), m_length (0), m_fw (1), m_bw (0)
{
  // .. nothing yet ..
}

void
ProgressBar::set_value (double v, const std::string &value)
{
  if (value != m_value_string) {
    update ();
    m_value_string = value;
  }

  m_value = v;
  int l = int (floor ((v < 0.0 ? 0.0 : v) * 0.01 * (double (m_width - 2) - 1e-6))) % m_width;
  if (l != m_length) {
    m_length = l;
    update ();
  }
}

void 
ProgressBar::paintEvent (QPaintEvent *event)
{
  QPainter painter (this);

  painter.fillRect (QRect (QPoint (m_fw, m_fw), QPoint (m_length + m_fw - 1, height () - 2 - m_fw)), palette ().brush (QPalette::Highlight));
  painter.fillRect (QRect (QPoint (m_length + m_fw, m_fw), QPoint (width () - m_fw, height () - 2 - m_fw)), palette ().brush (QPalette::Base));
  painter.setPen (palette ().color (QPalette::Text));
  
  for (int d = 0; d < m_bw; ++d) {
    painter.drawRect (QRect (QPoint (d, d), QPoint (width () - 2 - d, height () - 2 - d)));
  }

  painter.setFont (font ());

  painter.setClipRect (QRect (QPoint (m_fw, m_fw), QPoint (m_length + m_fw - 1, height () - 1 - m_fw)));
  painter.setPen (palette ().color (QPalette::HighlightedText));
  painter.drawText (rect (), Qt::AlignHCenter | Qt::AlignVCenter, m_value_string.c_str ());

  painter.setClipRect (QRect (QPoint (m_length + m_fw, 0), QPoint (width () - m_fw, height () - 1 - m_fw)));
  painter.setPen (palette ().color (QPalette::Text));
  painter.drawText (rect (), Qt::AlignHCenter | Qt::AlignVCenter, m_value_string.c_str ());
}

void
ProgressBar::resizeEvent (QResizeEvent *event)
{
  m_width = size ().width ();
  update ();
}

// --------------------------------------------------------------------

ProgressWidget::ProgressWidget (ProgressReporter *pr, QWidget *parent, const char *name)
  : QFrame (parent, name),
    mp_pr (pr)
{
  int button_size = 12;

  QHBoxLayout *layout = new QHBoxLayout (this);
  layout->setSpacing (4);
  layout->setMargin (0);

  layout->addSpacing (4);

  mp_label = new QLabel (this);
  layout->addWidget (mp_label, 1);

  mp_progress_bar = new ProgressBar (this);
  layout->addWidget (mp_progress_bar);

  mp_cancel_button = new QToolButton (this);
  mp_cancel_button->setMaximumSize (QSize (button_size, button_size));
  layout->addWidget (mp_cancel_button);

  layout->addSpacing (4);

  connect (mp_cancel_button, SIGNAL (clicked ()), this, SLOT (signal_break ()));
}

void 
ProgressWidget::set_text (const std::string &text)
{
  mp_label->setText (text.c_str ());
}

void 
ProgressWidget::set_value (double v, const std::string &value)
{
  mp_progress_bar->set_value (v, value);
}

void 
ProgressWidget::signal_break ()
{
  mp_pr->signal_break ();
}

// --------------------------------------------------------------------

//  HINT: this per-thread storage object is used to distinguish 
//  the main thread from the thread that called yield. It is 
//  assumed that only the thread that created the reporter object
//  may process events and manipulate the progress bar.
static QThreadStorage<int *> s_thread_data;

ProgressReporter::ProgressReporter (lay::StatusBar *sb, QApplication *app)
  : mp_app (app), m_start_time (0), mp_sb (sb), m_pw_visible (false)
{
  //  create a dummy int on the thread-local storage object to identify the main thread.
  s_thread_data.setLocalData (new int (1));
  tl::Progress::register_adaptor (this);
}

ProgressReporter::~ProgressReporter ()
{
  tl::Progress::register_adaptor (0);
  mp_sb = 0;
}

void 
ProgressReporter::register_object (tl::Progress *progress)
{
  if (! s_thread_data.hasLocalData ()) {
    //  not in main thread ..
    return;
  }

  mp_objects.push_front (progress);

  if (m_start_time == 0 && ! m_pw_visible) {
    m_start_time = clock ();
  }

  //  not supported yet: mp_sb->progress_widget ()->set_title (progress->title ());
  mp_sb->progress_widget ()->set_text (progress->desc ());
  mp_sb->progress_widget ()->set_value ((*mp_objects.begin ())->value (), (*mp_objects.begin ())->formatted_value ());
  //  make dialog visible after some time has passed
  if (! m_pw_visible && (clock () - m_start_time) > CLOCKS_PER_SEC) {
    mp_sb->show_progress_widget (true);
    m_pw_visible = true;
  }
  processEvents (); // Qt4 seems to need this
}

void 
ProgressReporter::unregister_object (tl::Progress *progress)
{
  if (! s_thread_data.hasLocalData ()) {
    //  not in main thread ..
    return;
  }

  for (std::list <tl::Progress *>::iterator k = mp_objects.begin (); k != mp_objects.end (); ++k) {
    if (*k == progress) {
      mp_objects.erase (k);
      //  close or refresh window
      if (mp_objects.empty ()) {
        mp_sb->show_progress_widget (false);
        m_pw_visible = false;
        m_start_time = 0;
      } else {
        //  not supported yet: mp_sb->progress_widget ()->set_title ((*mp_objects.begin ())->title ());
        mp_sb->progress_widget ()->set_text ((*mp_objects.begin ())->desc ());
        mp_sb->progress_widget ()->set_value ((*mp_objects.begin ())->value (), (*mp_objects.begin ())->formatted_value ());
      }
      processEvents (); // Qt4 seems to need this
      return;
    }
  }
}

void 
ProgressReporter::trigger (tl::Progress *progress)
{
  if (! s_thread_data.hasLocalData ()) {
    //  not in main thread ..
    return;
  }

  if (! mp_objects.empty () && mp_objects.front () == progress) {
    //  make dialog visible after some time has passed
    if (! m_pw_visible && (clock () - m_start_time) > CLOCKS_PER_SEC) {
      mp_sb->show_progress_widget (true);
      m_pw_visible = true;
    }
    mp_sb->progress_widget ()->set_text ((*mp_objects.begin ())->desc ());
    mp_sb->progress_widget ()->set_value ((*mp_objects.begin ())->value (), (*mp_objects.begin ())->formatted_value ());
    processEvents (); // Qt4 seems to need this
  }
}

void 
ProgressReporter::yield (tl::Progress *progress)
{
  if (! s_thread_data.hasLocalData ()) {
    //  not in main thread ..
    return;
  }

  //  make dialog visible after some time has passed
  if (! m_pw_visible && (clock () - m_start_time) > CLOCKS_PER_SEC) {
    mp_sb->show_progress_widget (true);
    m_pw_visible = true;
  }

  //  process events
  processEvents ();
}

void 
ProgressReporter::signal_break ()
{
  if (! mp_objects.empty ()) {
    (*mp_objects.begin ())->signal_break ();
  }
}

void
ProgressReporter::processEvents ()
{
  if (m_pw_visible) {
    QApplication::syncX ();
    mp_app->processEvents (QEventLoop::AllEvents);
  }
}

} // namespace lay

==> klayout-0.10.tar.gz: klayout-0.10/src/layProgress.cc <==

/*

  KLayout Layout Viewer
  Copyright (C) 2006 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include <QtCore/QMutex>
#include <QtCore/QThreadStorage>
#include <QtGui/QToolButton>
#include <QtGui/QLabel>
#include <QtGui/QPainter>
#include <QtGui/QHBoxLayout>
#include <QtGui/QApplication>

#include <stdlib.h>
#include <math.h>

#include <layProgress.h>
#include <layLayoutCanvas.h>
#include <tlProgress.h>

namespace lay
{

// --------------------------------------------------------------------

class ProgressBar
  : public QWidget 
{
public:
  ProgressBar (QWidget *parent, const char *name = "");

  void set_value (double v, const std::string &value);

  QSize sizeHint () const 
  {
    return QSize (m_width, 1);
  }

  QSize minimumSizeHint () const 
  {
    return QSize (m_width, 1);
  }

private:
  double m_value;
  std::string m_value_string;
  int m_width;
  int m_length;
  int m_fw;
  int m_bw;

  void paintEvent (QPaintEvent *event); 
  void resizeEvent (QResizeEvent *event); 
};

ProgressBar::ProgressBar (QWidget *parent, const char *name)
  : QWidget (parent, name),
    m_value (0.0), m_width (64), m_length (0), m_fw (1), m_bw (0)
{
  // .. nothing yet ..
}

void
ProgressBar::set_value (double v, const std::string &value)
{
  if (value != m_value_string) {
    update ();
    m_value_string = value;
  }

  m_value = v;
  int l = int (floor ((v < 0.0 ? 0.0 : v) * 0.01 * (double (m_width - 2) - 1e-6))) % m_width;
  if (l != m_length) {
    m_length = l;
    update ();
  }
}

void 
ProgressBar::paintEvent (QPaintEvent *event)
{
  QPainter painter (this);

  painter.fillRect (QRect (QPoint (m_fw, m_fw), QPoint (m_length + m_fw - 1, height () - 2 - m_fw)), palette ().brush (QPalette::Highlight));
  painter.fillRect (QRect (QPoint (m_length + m_fw, m_fw), QPoint (width () - m_fw, height () - 2 - m_fw)), palette ().brush (QPalette::Base));
  painter.setPen (palette ().color (QPalette::Text));
  
  for (int d = 0; d < m_bw; ++d) {
    painter.drawRect (QRect (QPoint (d, d), QPoint (width () - 2 - d, height () - 2 - d)));
  }

  painter.setFont (font ());

  painter.setClipRect (QRect (QPoint (m_fw, m_fw), QPoint (m_length + m_fw - 1, height () - 1 - m_fw)));
  painter.setPen (palette ().color (QPalette::HighlightedText));
  painter.drawText (rect (), Qt::AlignHCenter | Qt::AlignVCenter, m_value_string.c_str ());

  painter.setClipRect (QRect (QPoint (m_length + m_fw, 0), QPoint (width () - m_fw, height () - 1 - m_fw)));
  painter.setPen (palette ().color (QPalette::Text));
  painter.drawText (rect (), Qt::AlignHCenter | Qt::AlignVCenter, m_value_string.c_str ());
}

void
ProgressBar::resizeEvent (QResizeEvent *event)
{
  m_width = size ().width ();
  update ();
}

// --------------------------------------------------------------------

ProgressWidget::ProgressWidget (ProgressReporter *pr, QWidget *parent, const char *name)
  : QFrame (parent, name),
    mp_pr (pr)
{
  int button_size = 12;

  QHBoxLayout *layout = new QHBoxLayout (this);
  layout->setSpacing (4);
  layout->setMargin (0);

  layout->addSpacing (4);

  mp_label = new QLabel (this);
  layout->addWidget (mp_label, 1);

  mp_progress_bar = new ProgressBar (this);
  layout->addWidget (mp_progress_bar);

  mp_cancel_button = new QToolButton (this);
  mp_cancel_button->setMaximumSize (QSize (button_size, button_size));
  layout->addWidget (mp_cancel_button);

  layout->addSpacing (4);

  connect (mp_cancel_button, SIGNAL (clicked ()), this, SLOT (signal_break ()));
}

void 
ProgressWidget::set_text (const std::string &text)
{
  mp_label->setText (text.c_str ());
}

void 
ProgressWidget::set_value (double v, const std::string &value)
{
  mp_progress_bar->set_value (v, value);
}

void 
ProgressWidget::signal_break ()
{
  mp_pr->signal_break ();
}

// --------------------------------------------------------------------

//  HINT: this per-thread storage object is used to distinguish 
//  the main thread from the thread that called yield. It is 
//  assumed that only the thread that created the reporter object
//  may process events and manipulate the progress bar.
static QThreadStorage<int *> s_thread_data;

ProgressReporter::ProgressReporter (lay::StatusBar *sb, QApplication *app)
  : mp_app (app), m_start_time (), mp_sb (sb), m_pw_visible (false)
{
  //  create a dummy int on the thread-local storage object to identify the main thread.
  s_thread_data.setLocalData (new int (1));
  tl::Progress::register_adaptor (this);
}

ProgressReporter::~ProgressReporter ()
{
  tl::Progress::register_adaptor (0);
  mp_sb = 0;
}

void 
ProgressReporter::register_object (tl::Progress *progress)
{
  if (! s_thread_data.hasLocalData ()) {
    //  not in main thread ..
    return;
  }

  mp_objects.push_front (progress);

  if (m_start_time == tl::Clock () && ! m_pw_visible) {
    m_start_time = tl::Clock::current ();
  }

  //  not supported yet: mp_sb->progress_widget ()->set_title (progress->title ());
  mp_sb->progress_widget ()->set_text (progress->desc ());
  mp_sb->progress_widget ()->set_value ((*mp_objects.begin ())->value (), (*mp_objects.begin ())->formatted_value ());
  //  make dialog visible after some time has passed
  if (! m_pw_visible && (tl::Clock::current () - m_start_time).seconds () > 1.0) {
    mp_sb->show_progress_widget (true);
    m_pw_visible = true;
  }
  processEvents (); // Qt4 seems to need this
}

void 
ProgressReporter::unregister_object (tl::Progress *progress)
{
  if (! s_thread_data.hasLocalData ()) {
    //  not in main thread ..
    return;
  }

  for (std::list <tl::Progress *>::iterator k = mp_objects.begin (); k != mp_objects.end (); ++k) {
    if (*k == progress) {
      mp_objects.erase (k);
      //  close or refresh window
      if (mp_objects.empty ()) {
        mp_sb->show_progress_widget (false);
        m_pw_visible = false;
        m_start_time = tl::Clock ();
      } else {
        //  not supported yet: mp_sb->progress_widget ()->set_title ((*mp_objects.begin ())->title ());
        mp_sb->progress_widget ()->set_text ((*mp_objects.begin ())->desc ());
        mp_sb->progress_widget ()->set_value ((*mp_objects.begin ())->value (), (*mp_objects.begin ())->formatted_value ());
      }
      processEvents (); // Qt4 seems to need this
      return;
    }
  }
}

void 
ProgressReporter::trigger (tl::Progress *progress)
{
  if (! s_thread_data.hasLocalData ()) {
    //  not in main thread ..
    return;
  }

  if (! mp_objects.empty () && mp_objects.front () == progress) {
    //  make dialog visible after some time has passed
    if (! m_pw_visible && (tl::Clock::current () - m_start_time).seconds () > 1.0) {
      mp_sb->show_progress_widget (true);
      m_pw_visible = true;
    }
    mp_sb->progress_widget ()->set_text ((*mp_objects.begin ())->desc ());
    mp_sb->progress_widget ()->set_value ((*mp_objects.begin ())->value (), (*mp_objects.begin ())->formatted_value ());
    processEvents (); // Qt4 seems to need this
  }
}

void 
ProgressReporter::yield (tl::Progress *progress)
{
  if (! s_thread_data.hasLocalData ()) {
    //  not in main thread ..
    return;
  }

  //  make dialog visible after some time has passed
  if (! m_pw_visible && (tl::Clock::current () - m_start_time).seconds () > 1.0) {
    mp_sb->show_progress_widget (true);
    m_pw_visible = true;
  }

  //  process events
  processEvents ();
}

void 
ProgressReporter::signal_break ()
{
  if (! mp_objects.empty ()) {
    (*mp_objects.begin ())->signal_break ();
  }
}

void
ProgressReporter::processEvents ()
{
  if (m_pw_visible) {
    QApplication::syncX ();
    mp_app->processEvents (QEventLoop::AllEvents);
  }
}

} // namespace lay
